﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Windows.Input;

namespace App2010MenuKeyTips
{
    /// <summary>
    /// From MVVM article here
    /// http://msdn.microsoft.com/en-us/magazine/dd419663.aspx
    /// Command that allows for the injection of an action to be performed 
    /// and a function to determine if the command can execute. Use RelayCommand(T) when
    /// external values are needed for the Action and Predicate
    /// </summary>
    public class RelayCommand : ICommand
    {
        #region Fields
        private readonly Func<bool> canExecute;
        private readonly Action execute;
        #endregion // Fields

        public event EventHandler CanExecuteChanged
        {
            add { if (canExecute != null) CommandManager.RequerySuggested += value; }
            remove { if (canExecute != null) CommandManager.RequerySuggested -= value; }
        }
         public RelayCommand(Action execute)
            : this(execute, null)
        {
        }
        
        public RelayCommand(Action execute, Func<bool> canExecute)
        {
            if (execute == null)
                throw new ArgumentNullException("execute");

            this.execute = execute;
            this.canExecute = canExecute;
        }

        [DebuggerStepThrough]
        public bool CanExecute(object parameter)
        {
            return canExecute == null || canExecute();
        }

        public virtual void Execute(object parameter)
        {
            execute();
        }
    }

}
